@@ -1,8 +1,8 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "jwt" "0.2.0"
+(define-package "jwt" "20251022.2118"
   "Interact with JSON Web Tokens."
   '((emacs "29.1"))
   :url "https://github.com/joshbax189/jwt-el"
-  :commit "d7deb62f8c2df58d5cfebf087a147c75207964e8"
-  :revdesc "d7deb62f8c2d"
+  :commit "d6754f8fab6ff4041a7bece1963495e99ad9fe68"
+  :revdesc "d6754f8fab6f"
   :keywords '("tools" "convenience"))
