@@ -2,8 +2,8 @@
 
 ;; Author: Josh Bax
 ;; Maintainer: Josh Bax
-;; Package-Version: 0.2.0
-;; Package-Revision: d7deb62f8c2d
+;; Package-Version: 20251022.2118
+;; Package-Revision: d6754f8fab6f
 ;; Package-Requires: ((emacs "29.1"))
 ;; Homepage: https://github.com/joshbax189/jwt-el
 ;; Keywords: tools convenience
@@ -40,6 +40,11 @@
 (require 'cl-lib)
 (require 'hmac-def)
 (require 'calc-arith)
+(require 'rx)
+
+(defgroup jwt-el nil
+  "JSON Web Token display and signing."
+  :group 'comm)
 
 (defvar jwt--defined-claims-alist
   '(;; registered claim names
@@ -614,6 +619,92 @@ Specifically it checks that TEST-STRING has
            (doco (car maybe-doc)))
       (funcall callback doco :thing full-name))))
 
+(defcustom jwt-enable-overlays t
+  "If non-nil, display overlays."
+  :type 'boolean
+  :group 'jwt-el)
+
+(defun jwt--format-time (time)
+  "Make numeric TIME human readable."
+  (when (stringp time)
+    (setq time (string-to-number time)))
+  (format-time-string "%Y-%m-%d %a %H:%M:%S %Z" time))
+
+(defun jwt--make-claim-regexp (claim)
+  "Return a regexp matching a line with JSON property CLAIM.
+The value of the property is in the first capture group.
+Assumes JSON is formatted so that there is a single property per line."
+  (rx "\"" (literal claim) "\"" (* space) ":"
+      (* space) (? "\"") (group (+ word)) (? "\"") (? ",")))
+
+(defface jwt-annotation
+  '((default :box
+             (:line-width (1 . 1)
+                          :color nil
+                          :style nil))
+    (((class color) (background light)) :foreground "SkyBlue4")
+    (((class color) (background  dark)) :foreground "LightSkyBlue1"))
+  "Face for JWT claim annotation overlays."
+  :group 'jwt-el)
+
+(defface jwt-annotation-expired
+  '((t :inherit jwt-annotation :foreground "red" ))
+  "Face for expired overlay."
+  :group 'jwt-el)
+
+(defface jwt-annotation-not-valid
+  '((t :inherit jwt-annotation :foreground "orange" ))
+  "Face for overlay for when nbf time has not been reached."
+  :group 'jwt-el)
+
+(defvar jwt--annotation-claim-functions
+  (list
+   (list :regexp (jwt--make-claim-regexp "nbf")
+         :function (lambda ()
+                     (let* ((time (string-to-number (match-string-no-properties 1)))
+                            (formatted-time (jwt--format-time time)))
+                       (if (> time (float-time))
+                           (propertize (format "Not valid until %s" formatted-time) 'face 'jwt-annotation-not-valid)
+                         (propertize (format "Not before %s" formatted-time) 'face 'jwt-annotation)))))
+   (list :regexp (jwt--make-claim-regexp "iat")
+         :function (lambda ()
+                     (let* ((time (match-string-no-properties 1))
+                            (formatted-time (jwt--format-time time)))
+                       (propertize (format "Issued at %s" formatted-time) 'face 'jwt-annotation))))
+   (list :regexp (jwt--make-claim-regexp "exp")
+         :function (lambda ()
+                     (let* ((time (string-to-number (match-string-no-properties 1)))
+                            (formatted-time (jwt--format-time time)))
+                       (if (< time (float-time))
+                           (propertize (format "Expired %s" formatted-time) 'face 'jwt-annotation-expired)
+                         (propertize (format "Expires at %s" formatted-time) 'face 'jwt-annotation))))))
+  "List of forms like (:regexp R :function F).
+Function forms F should take no arguments and return a propertized string.
+Match data will be set with the result of matching R.")
+
+(defun jwt--annotation-add-overlays (beg end)
+  "Add JWT related overlays between BEG and END."
+  (save-excursion
+    (goto-char beg)
+    (dolist (claim-fn jwt--annotation-claim-functions)
+      (let ((claim-rx (plist-get claim-fn :regexp))
+            (claim-fn (plist-get claim-fn :function)))
+        (save-match-data
+          (while (re-search-forward claim-rx end t)
+            (let ((ov (make-overlay (match-beginning 0) (match-end 0))))
+              (overlay-put ov 'category 'jwt)
+              (overlay-put ov 'after-string (concat " " (funcall claim-fn))))))))))
+
+(defun jwt--annotation-remove-overlays (beg end)
+  "Cleanup all JWT related overlays between BEG and END."
+  (remove-overlays beg end 'category 'jwt))
+
+(defun jwt--update-overlays (beg end)
+  "Update JWT related overlays between BEG and END."
+  (jwt--annotation-remove-overlays beg end)
+  (when jwt-enable-overlays
+    (jwt--annotation-add-overlays beg end)))
+
 (define-minor-mode jwt-minor-mode
   "Display decoded contents of JWTs."
   :interactive nil
@@ -621,6 +712,7 @@ Specifically it checks that TEST-STRING has
             "C-c C-c" #'jwt-verify-current-token)
   :lighter " JWT-decoded"
   (add-hook 'eldoc-documentation-functions #'jwt--eldoc nil t)
+  (jit-lock-register 'jwt--update-overlays)
   (eldoc-mode))
 
 (provide 'jwt)
